#include "PlyFile.h"
#include "rply_int.h"
#include <cmath>

PlyFile::PlyFile()
{
	xmin=100000;
	xmax=-100000;
	ymin=100000;
	ymax=-100000;
	zmin=100000;
	zmax=-100000;
	init = -1;
}


PlyFile::~PlyFile()
{
}


bool PlyFile::readPLY(const char *filename)
{
	long nvertices, ntriangles;
	p_ply ply = ply_open(filename, NULL);
	if (!ply) return 1;
	if (!ply_read_header(ply)) return 1;
	nvertices = ply_set_read_cb(ply, "vertex", "x", vertex_x_cb,this, 0);
	ply_set_read_cb(ply, "vertex", "y",vertex_y_cb, this, 0);
	ply_set_read_cb(ply, "vertex", "z", vertex_z_cb, this, 0);
	
	ntriangles = ply_set_read_cb(ply, "face", "vertex_indices", face_cb, this, 0);

	if (!ply_read(ply)) return false;
	ply_close(ply);
	cout<<xmin<<" < x < "<<xmax<<endl;
	cout<<ymin<<" < y < "<<ymax<<endl;
	cout<<zmin<<" < z < "<<zmax<<endl;
	return true;
	
}

int PlyFile::convert(double rx,double ry,double rz)
{
	double bx,by,bz;
	bx=xmax-xmin;
	by=ymax-ymin;
	bz=zmax-zmin;
	Ox=(xmin+xmax)/2.0;
	Oy=(ymin+ymax)/2.0;
	Oz=(zmin+zmax)/2.0;
	double t=max(bx,max(by,bz));
	lx=-2*t;
	ly=0.5*t;
	lz=t;
	lx1=-2*t;
	ly1=-0.5*t;
	lz1=0;
	cx=-3*t;
	cy=0;
	cz=0;

	anglex=rx;
	angley=ry;
	anglez=rz;
}

int PlyFile::writePOV(const char *filename)
{
	Nvertices=(unsigned int)(vertices.size());
	Nfaces=(unsigned int)(faces.size());
	ofstream of;
	of.open(filename);
	of<<"global_settings {"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t assumed_gamma 2"<<endl;
	of<<"}"<<endl;
	of<<"background { color rgb <1.000000, 1.000000, 1.000000>}"<<endl;
	of<<endl;
	of<<"camera {"<<endl;
	of<<"\t perspective"<<endl;
	of<<"\t location <"<<cx<<","<<cy<<","<<cz<<">"<<endl;
	of<<"\t sky <0.000000, 0.000000, 1.000000>"<<endl;
	of<<"\t right <0, 1, 0>"<<endl;
	of<<"\t angle "<<30<<endl;
	of<<"\t look_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx<<","<<ly<<","<<lz<<">"<<endl;
	of<<"\t color <1,1,1>*0.5"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <0,2,-1>,<2.5,1,-2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx1<<","<<ly1<<","<<lz1<<">"<<endl;
	of<<"\t color <1,1,1>*0.25"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <-1,4,0>,<0,0,9/2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"mesh2 {"<<endl;
	of<<"\t vertex_vectors {"<<endl;
	of<<"\t \t"<<Nvertices<<","<<endl;
	for(deque<Pt>::iterator it=vertices.begin();it!=vertices.end();it++)
	{
		of<<"\t \t <"<<it->x-Ox<<","<<it->y-Oy<<","<<it->z-Oz<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t face_indices {"<<endl;
	of<<"\t \t"<<Nfaces<<","<<endl;
	for(deque<Face>::iterator it=faces.begin();it!=faces.end();it++)
	{
		of<<"\t \t <"<<it->a<<","<<it->b<<","<<it->c<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t matrix < 1,0,0,"<<endl;
	of<<"\t 0,1,0,"<<endl;
	of<<"\t	 0,0,1"<<endl;
	of<<"\t 0, 0, 0>"<<endl;
	of<<"\t texture {"<<endl;
	of<<"\t \t pigment {"<<endl;
	of<<"\t \t color rgbf <1.000000, 1.000000, 1.000000 0.000000>"<<endl;
	of<<"\t	}"<<endl;
	of<<"\t \t finish {"<<endl;
	of<<"\t	\t \t ambient 0.000000  diffuse 1.000000  phong 0.100000  phong_size 100.000000"<<endl;
	of<<"\t	\t }"<<endl;
	of<<"\t }"<<endl;
	if(abs(anglex)>0.0001)
		of<<"\t rotate "<<anglex<<"*x"<<endl;
	if(abs(angley)>0.0001)
		of<<"\t rotate "<<angley<<"*y"<<endl;
	if(abs(anglez)>0.0001)
		of<<"\t rotate "<<anglez<<"*z"<<endl;
	of<<"}"<<endl;
	of.close();
}

int PlyFile::writePOVheader(const char* headerfile, const char* mesh2file)
{
	ofstream of;
	of.open(headerfile);
	of<<"global_settings {"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t assumed_gamma 2"<<endl;
	of<<"}"<<endl;
	of<<"background { color rgb <1.000000, 1.000000, 1.000000>}"<<endl;
	of<<endl;
	of<<"camera {"<<endl;
	of<<"\t perspective"<<endl;
	of<<"\t location <"<<cx<<","<<cy<<","<<cz<<">"<<endl;
	of<<"\t sky <0.000000, 0.000000, 1.000000>"<<endl;
	of<<"\t right <0, 1, 0>"<<endl;
	of<<"\t angle "<<30<<endl;
	of<<"\t look_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx<<","<<ly<<","<<lz<<">"<<endl;
	of<<"\t color <1,1,1>*0.5"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <0,2,-1>,<2.5,1,-2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx1<<","<<ly1<<","<<lz1<<">"<<endl;
	of<<"\t color <1,1,1>*0.25"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <-1,4,0>,<0,0,9/2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;

	of<<"#declare mymesh ="<<endl;
	of<<"#include \""<<mesh2file <<"\""<<endl;

	of.close();
}


int PlyFile::writeAnimPOV(const char *filename)
{
	Nvertices=(unsigned int)(vertices.size());
	Nfaces=(unsigned int)(faces.size());
	ofstream of;
	of.open(filename);
	of<<"global_settings {"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t ambient_light color rgb <1.0, 1.0, 1.0>"<<endl;
	of<<"\t assumed_gamma 2"<<endl;
	of<<"}"<<endl;
	of<<"background { color rgb <1.000000, 1.000000, 1.000000>}"<<endl;
	of<<endl;
	of<<"camera {"<<endl;
	of<<"\t perspective"<<endl;
	of<<"\t location <"<<cx<<","<<cy<<","<<cz<<">"<<endl;
	of<<"\t sky <0.000000, 0.000000, 1.000000>"<<endl;
	of<<"\t right <0, 1, 0>"<<endl;
	of<<"\t angle "<<30<<endl;
	of<<"\t look_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx<<","<<ly<<","<<lz<<">"<<endl;
	of<<"\t color <1,1,1>*0.5"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <0,2,-1>,<2.5,1,-2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"light_source {"<<endl;
	of<<"\t <"<<lx1<<","<<ly1<<","<<lz1<<">"<<endl;
	of<<"\t color <1,1,1>*0.25"<<endl;
	of<<"\t parallel"<<endl;
	of<<"\t point_at <"<<0<<","<<0<<","<<0<<">"<<endl;
	of<<"\t area_light"<<endl;
	of<<"\t <-1,4,0>,<0,0,9/2>4,4"<<endl;
	of<<"\t adaptive 1"<<endl;
	of<<"}"<<endl;
	of<<endl;
	of<<"mesh2 {"<<endl;
	of<<"\t vertex_vectors {"<<endl;
	of<<"\t \t"<<Nvertices<<","<<endl;
	for(deque<Pt>::iterator it=vertices.begin();it!=vertices.end();it++)
	{
		of<<"\t \t <"<<it->x-Ox<<","<<it->y-Oy<<","<<it->z-Oz<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t face_indices {"<<endl;
	of<<"\t \t"<<Nfaces<<","<<endl;
	for(deque<Face>::iterator it=faces.begin();it!=faces.end();it++)
	{
		of<<"\t \t <"<<it->a<<","<<it->b<<","<<it->c<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t matrix < 1,0,0,"<<endl;
	of<<"\t 0,1,0,"<<endl;
	of<<"\t	 0,0,1"<<endl;
	of<<"\t 0, 0, 0>"<<endl;
	of<<"\t texture {"<<endl;
	of<<"\t \t pigment {"<<endl;
	of<<"\t \t color rgbf <1.000000, 1.000000, 1.000000 0.000000>"<<endl;
	of<<"\t	}"<<endl;
	of<<"\t \t finish {"<<endl;
	of<<"\t	\t \t ambient 0.000000  diffuse 1.000000  phong 0.100000  phong_size 100.000000"<<endl;
	of<<"\t	\t }"<<endl;
	of<<"\t }"<<endl;
	of<<"rotate clock*z*180"<<endl;
	of<<"}"<<endl;
	of.close();
}

int PlyFile::writeMesh2(const char *filename)
{
	Nvertices=(unsigned int)(vertices.size());
	Nfaces=(unsigned int)(faces.size());
	ofstream of;
	of.open(filename);
	of<<"mesh2 {"<<endl;
	of<<"\t vertex_vectors {"<<endl;
	of<<"\t \t"<<Nvertices<<","<<endl;
	for(deque<Pt>::iterator it=vertices.begin();it!=vertices.end();it++)
	{
		of<<"\t \t <"<<it->x-Ox<<","<<it->y-Oy<<","<<it->z-Oz<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t face_indices {"<<endl;
	of<<"\t \t"<<Nfaces<<","<<endl;
	for(deque<Face>::iterator it=faces.begin();it!=faces.end();it++)
	{
		of<<"\t \t <"<<it->a<<","<<it->b<<","<<it->c<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t matrix < 1,0,0,"<<endl;
	of<<"\t 0,1,0,"<<endl;
	of<<"\t	 0,0,1"<<endl;
	of<<"\t 0, 0, 0>"<<endl;
	of<<"\t texture {"<<endl;
	of<<"\t \t pigment {"<<endl;
	of<<"\t \t color rgbf <1.000000, 1.000000, 1.000000 0.000000>"<<endl;
	of<<"\t	}"<<endl;
	of<<"\t \t finish {"<<endl;
	of<<"\t	\t \t ambient 0.000000  diffuse 1.000000  phong 0.100000  phong_size 100.000000"<<endl;
	of<<"\t	\t }"<<endl;
	of<<"\t }"<<endl;
	of<<"}"<<endl;
	of.close();
}

int PlyFile::writeMesh2Core(const char *filename)
{
	Nvertices=(unsigned int)(vertices.size());
	Nfaces=(unsigned int)(faces.size());
	ofstream of;
	of.open(filename);
	of<<"\t vertex_vectors {"<<endl;
	of<<"\t \t"<<Nvertices<<","<<endl;
	for(deque<Pt>::iterator it=vertices.begin();it!=vertices.end();it++)
	{
		of<<"\t \t <"<<it->x<<","<<it->y<<","<<it->z<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of<<"\t face_indices {"<<endl;
	of<<"\t \t"<<Nfaces<<","<<endl;
	for(deque<Face>::iterator it=faces.begin();it!=faces.end();it++)
	{
		of<<"\t \t <"<<it->a<<","<<it->b<<","<<it->c<<">,"<<endl;
	}
	of<<"\t }"<<endl;
	of.close();
}
