#ifndef PLYFILE_H
#define PLYFILE_H


#include <iostream>
#include <cstdlib>
#include <fstream>
#include <deque>
#include "utilities.h"
#include "rply_int.h"
using namespace std;

/**Class providing the structure for a ply file*/
class PlyFile
{
	public :
	/**Constructor*/
	PlyFile();
			
	/**Destructor*/
	~PlyFile();

		
	/**read a file and build the mesh structure
	@param filename file to parse
	*/
	bool readPLY(const char *filename);

	/**convert to mesh2 structure*/
	int convert(double rx,double ry,double rz);
	
	/**write pov file*/
	int writePOV(const char *filename);
	
	/**write povray header*/
	int writePOVheader(const char *headerfile,const char *mesh2file);

	/**write pov file for animation (rotation around z axis*/
	int writeAnimPOV(const char *filename);

	/**write file containing only the mesh2 structure*/
	int writeMesh2(const char *filename);

	/**write Mesh2 Core file (only points and faces)*/
	int writeMesh2Core(const char *filename);
	
	/**temporary point*/
	Pt pt;
	
	/**Temporary face*/
	Face f;
	
	int n;
	
	deque<Pt> vertices;
	deque<Face> faces;

	/**min/max values for point coordinates*/
	double xmin,ymin,zmin;
	double xmax,ymax,zmax;

	/**Origin of the bounding box*/
	double Ox,Oy,Oz;

	/**Camera position*/
	double cx,cy,cz;

	/**Light position*/
	double lx,ly,lz;

	/**Light position*/
	double lx1,ly1,lz1;
	
	/**nber of points*/
	unsigned int Nvertices;

	/**nber of faces*/
	unsigned int Nfaces;

	/**rotate around x axis*/
	double anglex;

/**rotate around y axis*/
	double angley;

/**rotate around z axis*/
	double anglez;
	
	/**for the initialization*/
	int init;
	
};

/**vertex x property callback
@param argument p_ply_argument
@return 1 if ok
 */
static int vertex_x_cb(p_ply_argument argument)
{
	int current_cmpnt = 0;
	void* pointer_usrdata = NULL;
	ply_get_argument_user_data(argument, &pointer_usrdata, &current_cmpnt);
	PlyFile* pf = static_cast<PlyFile*>(pointer_usrdata);
	pf->pt.x=ply_get_argument_value(argument);
	if(pf->init == -1)
	{
	    pf->xmax = pf->pt.x;
	    pf->xmin = pf->pt.x;
	}
	else
	{
	if(pf->pt.x>pf->xmax)
		pf->xmax=pf->pt.x;
	else if(pf->pt.x<pf->xmin)
		pf->xmin=pf->pt.x;
	}
	return 1;
}

/**vertex y property callback
@param argument p_ply_argument
@return 1 if ok
 */
static int vertex_y_cb(p_ply_argument argument)
{
	int current_cmpnt = 0;
	void* pointer_usrdata = NULL;
	ply_get_argument_user_data(argument, &pointer_usrdata, &current_cmpnt);
	PlyFile* pf = static_cast<PlyFile*>(pointer_usrdata);
	pf->pt.y=ply_get_argument_value(argument);
	if(pf->init == -1)
	{
	    pf->ymax = pf->pt.y;
	    pf->ymin = pf->pt.y;
	}
	else
	{
	if(pf->pt.y>pf->ymax)
		pf->ymax=pf->pt.y;
	else if(pf->pt.y<pf->ymin)
		pf->ymin=pf->pt.y;
	}
	return 1;
}

/**vertex z property callback
@param argument p_ply_argument
@return 1 if ok
 */
static int vertex_z_cb(p_ply_argument argument)
{
	int current_cmpnt = 0;
	void* pointer_usrdata = NULL;
	ply_get_argument_user_data(argument, &pointer_usrdata, &current_cmpnt);
	PlyFile* pf = static_cast<PlyFile*>(pointer_usrdata);
	pf->pt.z=ply_get_argument_value(argument);
	if(pf->init == -1)
	{
	    pf->zmax = pf->pt.z;
	    pf->zmin = pf->pt.z;
	    pf->init = 1;
	}
	else
	{
	if(pf->pt.z>pf->zmax)
		pf->zmax=pf->pt.z;
	else if(pf->pt.z<pf->zmin)
		pf->zmin=pf->pt.z;
	}
	pf->vertices.push_back(pf->pt);
	return 1;
}



/**face callback
@param argument p_ply_argument
@return 1 if ok
 */

static int face_cb(p_ply_argument argument)
{
	int length, value_index;
	int current_cmpnt = 0;
	void* pointer_usrdata = NULL;
	ply_get_argument_property(argument,NULL, &length, &value_index);
	ply_get_argument_user_data(argument,&pointer_usrdata,&current_cmpnt);
	PlyFile* pf = static_cast<PlyFile*>(pointer_usrdata);
	
	switch (value_index) {
		case 0:
			pf->f.a=ply_get_argument_value(argument);
			break;
		case 1: 
			pf->f.b=ply_get_argument_value(argument);
			break;
		case 2:
			pf->f.c=ply_get_argument_value(argument);
			pf->faces.push_back(pf->f);
			break;
		default: 
			break;
	}
	return 1;
}


#endif

