/***************************************************************************
 *   Copyright (C) 2010 by Julie Digne   *
 *   julie.digne@cmla.ens-cachan.fr   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <getopt.h>
#include <sstream>
#include <ctime>
#include "PlyFile.h"

using namespace std;

int main(int argc, char *argv[])
{
  int c;
  stringstream f;
  string infile;
  string outfile;
  int mesh2_flag=-1;
  int core_mesh2_flag=-1;
  int anim_flag=-1;
  char a='a';
  double rx,ry,rz;
  rx=ry=rz=0;
  time_t start,end;
  
  while ((c = getopt (argc, argv, "i:o:pmcx:y:z:a")) != -1)
	{
		switch(c)
		{
			case 'i'://infile
				f<<optarg;
				f>>infile;
				cout<<infile<<endl;
				break;
			case 'o'://outfile
				outfile=optarg;
				break;
			case 'm'://save only mesh2 structure
				mesh2_flag=1;
				break;
			case 'c':
				core_mesh2_flag=1;
				break;
			case 'x':
				f.clear();
				f<<optarg;
				f>>rx;
				break;
			case 'y':
				f.clear();
				f<<optarg;
				f>>ry;
				break;
			case 'z':
				f.clear();
				f<<optarg;
				f>>rz;
				break;
			case 'a':
				anim_flag=1;
				break;
		}
	}
	
	
  std::time(&start);
  PlyFile *pf=new PlyFile();
  bool ok = pf->readPLY(infile.c_str());
  
  if(!ok)
    return EXIT_FAILURE;
	
  pf->convert(rx,ry,rz);
  if(mesh2_flag==1)
  {
	
	cout<<"write mesh2 and separate povray  file "<<endl;
	string mesh2file = outfile;
	mesh2file.append(".mesh2");
	pf->writeMesh2(mesh2file.c_str());
	string headerfile = outfile;
	headerfile.append(".pov");
	pf->writePOVheader(headerfile.c_str(), mesh2file.c_str());
  }
  else if (core_mesh2_flag==1)
	pf->writeMesh2Core(outfile.c_str());
  else
  { 
   if(anim_flag==1)
	pf->writeAnimPOV(outfile.c_str());
  else
	{
	cout<<"write povray file "<<endl;
   	pf->writePOV(outfile.c_str());
	}
  }

  delete pf;
  
  std::time(&end);
  std::cout<<"Total computation time: "<<difftime(end,start)<<" s."<<std::endl;
  
  return EXIT_SUCCESS;
}
